@@ -6,7 +6,7 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '2.008';
+our $VERSION = '2.009';
 
 use File::Basename qw( basename );
 use List::Util     qw( none any );
@@ -40,7 +40,7 @@ sub delete_table {
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $lyt_3 = Term::Choose->new( $sf->{i}{lyt_3} );
     my $sql = {};
-    my $prompt = '"' . basename( $sf->{d}{db} ) . '"' . "\n" . 'Drop table';
+    my $prompt = $sf->{d}{db_string} . "\n" . 'Drop table';
     # Choose
     my $table = $lyt_3->choose( #
         [ undef, map { "- $_" } @{$sf->{d}{user_tables}} ],
@@ -58,7 +58,7 @@ sub delete_table {
     my $row_count = @$all_arrayref;
     unshift @$all_arrayref, $col_names;
     if ( @$all_arrayref > 1 ) {
-        my $prompt_pt = "i: table to be deleted: $sql->{table}\n";
+        my $prompt_pt = "Table to be deleted: $sql->{table}\n";
         print_table( $all_arrayref, { %{$sf->{o}{table}}, prompt => $prompt_pt, max_rows => 0, keep_header => 0 } ); #
     }
     $prompt = sprintf 'DROP TABLE %s  (%d %s)', $sql->{table}, $row_count, $row_count == 1 ? 'row' : 'rows';
@@ -76,48 +76,17 @@ sub delete_table {
 }
 
 
-sub __table_name {
-    my ( $sf, $sql, $stmt_typeS ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $table;
-    my $c = 0;
-
-    TABLENAME: while ( 1 ) {
-        my $trs = Term::Form->new( 'tn' );
-        my $info;
-        if ( defined $sf->{d}{file_name} ) {
-            $info = sprintf "File: '%s'\n", basename( delete $sf->{d}{file_name} );
-        }
-        # Readline
-        $table = $trs->readline( 'Table name: ', { info => $info } );
-        if ( ! length $table ) {
-            return;
-        }
-        my $tmp_td = [ undef, $sf->{d}{schema}, $table ];
-        $sql->{table} = $ax->quote_table( $tmp_td );
-        if ( none { $sql->{table} eq $ax->quote_table( $sf->{d}{tables_info}{$_} ) } keys %{$sf->{d}{tables_info}} ) {
-            return 1;
-        }
-        $ax->print_sql( $sql, $stmt_typeS );
-        my $prompt = "Table $sql->{table} already exists.";
-        my $choice = choose(
-            [ undef, 'New name' ],
-            { %{$sf->{i}{lyt_stmt_h}}, prompt => $prompt, undef => 'BACK', layout => 3, justify => 0 }
-        );
-        if ( ! defined $choice ) {
-            return;
-        }
-    }
-}
 
 
-sub __reset_create_table_sql {
+sub __reset_create_table_sql { ###
     my ( $sf, $sql ) = @_;
     $sql->{table} = undef;
     $sql->{insert_into_args} = [];
     $sql->{insert_into_cols} = [];
+    $sql->{create_table_cols} = [];
 }
 
+
 sub create_new_table {
     my ( $sf ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
@@ -158,92 +127,58 @@ sub create_new_table {
             $opt->config_insert();
             next MENU;
         }
-        $ax->reset_sql( $sql ); #
+        $ax->reset_sql( $sql ); ##
+        $sf->{auto_inc_col_name} = $sf->{d}{driver} eq 'SQLite' ? $sf->{o}{create}{auto_inc_col_name} : '';
         if ( $custom eq $cu{create_table_plain} ) {
-            my $ok = $sf->__data_from_plain( $sql, $stmt_typeS );
+            $sql->{insert_into_args} = [];
+        }
+        elsif ( $custom eq $cu{create_table_form_copy} ) {
+            push @$stmt_typeS, 'Insert';
+            my $tbl_in = App::DBBrowser::Table::Insert->new( $sf->{i}, $sf->{o}, $sf->{d} );
+            my $ok = $tbl_in->from_copy_and_paste( $sql, $stmt_typeS );
             if ( ! $ok ) {
                 $sf->__reset_create_table_sql( $sql );
                 next MENU;
             }
         }
-        else {
-            if ( $custom eq $cu{create_table_form_copy} ) {
-                push @$stmt_typeS, 'Insert';
-                my $tbl_in = App::DBBrowser::Table::Insert->new( $sf->{i}, $sf->{o}, $sf->{d} );
-                my $ok = $tbl_in->from_copy_and_paste( $sql, $stmt_typeS );
-                if ( ! $ok ) {
-                    $sf->__reset_create_table_sql( $sql );
-                    next MENU;
-                }
-            }
-            elsif ( $custom eq $cu{create_table_form_file} ) {
-                push @$stmt_typeS, 'Insert';
-                my $tbl_in = App::DBBrowser::Table::Insert->new( $sf->{i}, $sf->{o}, $sf->{d} );
-                my $file_name = $tbl_in->from_file( $sql, $stmt_typeS );
-                if ( ! $file_name ) {
-                    $sf->__reset_create_table_sql( $sql );
-                    next MENU;
-                }
-                $sf->{d}{file_name} = $file_name;
-            }
-            my $ok = $sf->__table_name( $sql, $stmt_typeS );
-            if ( ! $ok ) {
+        elsif ( $custom eq $cu{create_table_form_file} ) {
+            push @$stmt_typeS, 'Insert';
+            my $tbl_in = App::DBBrowser::Table::Insert->new( $sf->{i}, $sf->{o}, $sf->{d} );
+            my $file_name = $tbl_in->from_file( $sql, $stmt_typeS );
+            if ( ! $file_name ) {
                 $sf->__reset_create_table_sql( $sql );
                 next MENU;
             }
-            # Columns
-            my ( $first_row, $user_input ) = ( '- First row', '- Add row' );
-            $ax->print_sql( $sql, $stmt_typeS );
-            # Choose
-            my $choice = choose(
-                [ undef, $first_row, $user_input ],
-                { %{$sf->{i}{lyt_stmt_v}}, prompt => 'Header:' }
-            );
-            if ( ! defined $choice ) {
-                $sf->__reset_create_table_sql( $sql );
-                next MENU;
-            }
-            if ( $choice eq $first_row ) {
-                $sql->{insert_into_cols} = shift @{$sql->{insert_into_args}}; # not quoted
-            }
-            else {
-                my $c = 0;
-                $sql->{insert_into_cols} = [ map { 'c' . ++$c } @{$sql->{insert_into_args}->[0]} ]; # not quoted
-            }
-            my $trs = Term::Form->new( 'cols' );
-            $ax->print_sql( $sql, $stmt_typeS );
-            # Fill_form
-            my $c = 0;
-            my $form = $trs->fill_form(
-                [ map { [ ++$c, defined $_ ? "$_" : '' ] } @{$sql->{insert_into_cols}} ], #
-                { prompt => 'Col names:', auto_up => 2, confirm => '  CONFIRM', back => '  BACK   ' }
-            );
-            if ( ! $form ) {
-                $sf->__reset_create_table_sql( $sql );
-                next MENU;
-            }
-            $sql->{insert_into_cols} = [ map { $_->[1] } @$form ]; # not quoted
+            $sf->{d}{file_name} = $file_name;
         }
-        die "Column with no name!" if any { ! length } @{$sql->{insert_into_cols}};
-        my @cols = @{$sql->{insert_into_cols}}; #
-        $sql->{insert_into_cols} = $ax->quote_simple_many( $sql->{insert_into_cols} );
-        # Datatypes
-        my $trs = Term::Form->new( 'cols' );
-        $ax->print_sql( $sql, $stmt_typeS );
-        # Fill_form
-        my $col_name_and_type = $trs->fill_form( # look
-            [ map { [ $_, $sf->{o}{insert}{default_data_type} ] } @cols ],
-            { prompt => 'Data types:', auto_up => 2, confirm => 'CONFIRM', back => 'BACK        ' }
-        );
-        if ( ! $col_name_and_type ) {
+
+        my $ok_table_name = $sf->__set_table_name( $sql, $stmt_typeS );
+        if ( ! $ok_table_name ) {
             $sf->__reset_create_table_sql( $sql );
             next MENU;
         }
-        my $qt_table = $sql->{table};
-        for my $i ( 0 .. $#{$sql->{insert_into_cols}} ) {
-            $sql->{create_table_cols}[$i] = $sql->{insert_into_cols}[$i] . ' ' . $col_name_and_type->[$i][1];
+        $ax->print_sql( $sql, $stmt_typeS );
+
+        my $ok_columns = $sf->__set_columns( $sql, $stmt_typeS );
+        if ( ! $ok_columns ) {
+            $sf->__reset_create_table_sql( $sql );
+            next MENU;
         }
+        if ( any { ! length } @{$sql->{insert_into_cols}} ) {
+            die "Column with no name!";
+        }
+        $sql->{insert_into_cols} = $ax->quote_simple_many( $sql->{insert_into_cols} );
+        $sql->{create_table_cols} = [ @{$sql->{insert_into_cols}} ];
+        $ax->print_sql( $sql, $stmt_typeS );
+
+        my $ok_data_types = $sf->__set_data_types( $sql, $stmt_typeS );
+        if ( ! $ok_data_types ) {
+            $sf->__reset_create_table_sql( $sql );
+            next MENU;
+        }
+
         # Create table
+        my $qt_table = $sql->{table};
         $ax->print_sql( $sql, $stmt_typeS );
         # Choose
         my $create_table_ok = choose(
@@ -263,6 +198,9 @@ sub create_new_table {
             $stmt_typeS = [ $stmt_typeS->[-1] ];
             my @columns = @{$sth->{NAME}};
             $sth->finish();
+            if ( length $sf->{auto_inc_col_name} ) {
+                shift @columns;
+            }
             $sql->{insert_into_cols} = $ax->quote_simple_many( \@columns );
             my $tbl = App::DBBrowser::Table->new( $sf->{i}, $sf->{o}, $sf->{d} );
             my $commit_ok = $tbl->commit_sql( $sql, $stmt_typeS );
@@ -272,35 +210,140 @@ sub create_new_table {
 }
 
 
-sub __data_from_plain {
+sub __set_table_name {
     my ( $sf, $sql, $stmt_typeS ) = @_;
-    my $ok = $sf->__table_name( $sql, $stmt_typeS );
-    if ( ! $ok ) {
-        return;
-    }
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    $ax->print_sql( $sql, $stmt_typeS );
-    my $col_count = choose_a_number( 3, { small => 1, confirm => 'Confirm', mouse => $sf->{o}{table}{mouse},
-                                          back => 'Back', name => 'Number of columns: ', clear_screen => 0 } );
-    if ( ! $col_count ) {
-        return;
+    my $table;
+    my $c = 0;
+
+    TABLENAME: while ( 1 ) {
+        my $trs = Term::Form->new( 'tn' );
+        my $info;
+        my $default;
+        if ( defined $sf->{d}{file_name} ) {
+            my $file = basename delete $sf->{d}{file_name};
+            $info = sprintf "File: '%s'\n", $file;
+            ( $default = $file ) =~ s/\.[^.]{1,3}\z//;
+        }
+        # Readline
+        $table = $trs->readline( 'Table name: ', { info => $info, default => $default } );
+        if ( ! length $table ) {
+            return;
+        }
+        my $tmp_td = [ undef, $sf->{d}{schema}, $table ];
+        $sql->{table} = $ax->quote_table( $tmp_td );
+        if ( none { $sql->{table} eq $ax->quote_table( $sf->{d}{tables_info}{$_} ) } keys %{$sf->{d}{tables_info}} ) {
+            return 1;
+        }
+        $ax->print_sql( $sql, $stmt_typeS );
+        my $prompt = "Table $sql->{table} already exists.";
+        my $choice = choose(
+            [ undef, 'New name' ],
+            { %{$sf->{i}{lyt_stmt_h}}, prompt => $prompt, undef => 'BACK', layout => 3, justify => 0 }
+        );
+        if ( ! defined $choice ) {
+            return;
+        }
     }
-    $ax->print_sql( $sql, $stmt_typeS );
-    my $info = 'Enter column names:';
-    my $trs = Term::Form->new();
-    my $col_names = $trs->fill_form(
-        [ map { [ $_, ] } 1 .. $col_count ],
-        { info => $info, confirm => 'OK', back => '<<', auto_up => 2 }
-    );
-    if ( ! defined $col_names ) {
-        return;
+}
+
+
+sub __set_columns {
+    my ( $sf, $sql, $stmt_typeS ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    if ( ! @{$sql->{insert_into_args}} ) {
+        $ax->print_sql( $sql, $stmt_typeS );
+        my $col_count = choose_a_number( 3, { small => 1, confirm => 'Confirm', mouse => $sf->{o}{table}{mouse},
+                                            back => 'Back', name => 'Number of columns: ', clear_screen => 0 } );
+        if ( ! $col_count ) {
+            return;
+        }
+        $ax->print_sql( $sql, $stmt_typeS );
+        my $info = 'Enter column names:';
+        my $trs = Term::Form->new();
+        my $col_names = $trs->fill_form(
+            [ map { [ $_, ] } 1 .. $col_count ],
+            { info => $info, confirm => 'OK', back => '<<', auto_up => 2 }
+        );
+        if ( ! defined $col_names ) {
+            return;
+        }
+        $sql->{insert_into_cols} = [ map { $_->[1] } @$col_names ]; # not quoted
+    }
+    else {
+        my ( $first_row, $user_input ) = ( '- First row', '- Add row' );
+        $ax->print_sql( $sql, $stmt_typeS );
+        # Choose
+        my $choice = choose(
+            [ undef, $first_row, $user_input ],
+            { %{$sf->{i}{lyt_stmt_v}}, prompt => 'Header:' }
+        );
+        if ( ! defined $choice ) {
+            $sf->__reset_create_table_sql( $sql );
+            return;
+        }
+        if ( $choice eq $first_row ) {
+            $sql->{insert_into_cols} = shift @{$sql->{insert_into_args}}; # not quoted
+        }
+        else {
+            my $c = 0;
+            $sql->{insert_into_cols} = [ map { 'c' . ++$c } @{$sql->{insert_into_args}->[0]} ]; # not quoted
+        }
+        my $c = 0;
+        my $fields = [ map { [ ++$c, defined $_ ? "$_" : '' ] } @{$sql->{insert_into_cols}} ];
+        #if ( $sf->{d}{driver} eq 'SQLite' ) {
+        if ( length $sf->{auto_inc_col_name} ) {
+            unshift @$fields, [ 'ai', $sf->{auto_inc_col_name} ];
+        }
+        my $trs = Term::Form->new( 'cols' );
+        $ax->print_sql( $sql, $stmt_typeS );
+        # Fill_form
+        my $form = $trs->fill_form(
+            $fields,
+            { prompt => 'Col names:', auto_up => 2, confirm => '  CONFIRM', back => '  BACK   ' }
+        );
+        if ( ! $form ) {
+            $sf->__reset_create_table_sql( $sql );
+            return;
+        }
+        #if ( $sf->{d}{driver} eq 'SQLite' ) {
+        if ( length $sf->{auto_inc_col_name} ) {
+            my $auto_col = shift @$form;
+            $sf->{auto_inc_col_name} = $auto_col->[1];
+        }
+        $sql->{insert_into_cols} = [ map { $_->[1] } @$form ]; # not quoted
     }
-    $sql->{insert_into_cols} = [ map { $_->[1] } @$col_names ]; # not quoted
-    $ax->print_sql( $sql, $stmt_typeS );
     return 1;
 }
 
 
+sub __set_data_types {
+    my ( $sf, $sql, $stmt_typeS ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $fields = [ map { [ $_, $sf->{o}{create}{default_data_type} ] } @{$sql->{create_table_cols}} ];
+    my $read_only = [];
+    if ( length $sf->{auto_inc_col_name} ) {
+        unshift @$fields, [ $ax->quote_col_qualified( [ $sf->{auto_inc_col_name} ] ), 'INTEGER PRIMARY KEY' ];
+        $read_only = [ 0 ];
+    }
+    my $trs = Term::Form->new( 'cols' );
+    $ax->print_sql( $sql, $stmt_typeS );
+    # Fill_form
+    my $col_name_and_type = $trs->fill_form( # look
+        $fields,
+        { prompt => 'Data types:', auto_up => 2, read_only => $read_only,
+            confirm => 'CONFIRM', back => 'BACK        ' }
+    );
+    if ( ! $col_name_and_type ) {
+        $sf->__reset_create_table_sql( $sql );
+        return;
+    }
+    $sql->{create_table_cols} = [ map { join ' ', @$_ }  @$col_name_and_type ];
+    return 1;
+}
+
+
+
 
 
 1;
